/******************************************************************************
 * BinaryModes.c
 *
 * Created on: May 25, 2012
 * Board: DRV2604EVK-CT, DRV2605EVK-CT
 * Author: a0866685
 *
 * Desc: This file contains the binary mode functions and effects. Include this
 * 		file to compile with extra modes.
 *
 ******************************************************************************/

#ifndef BINARYMODES_C_
#define BINARYMODES_C_

#include "BinaryModes.h"
#include "DRV2605.h"
#include "Actuator_Waveforms.h"
#include "CapTouchBoard.h"
#include "Recorder.h"

extern struct Element* modePtr;
extern struct Element* buttonPtr;
extern struct Element* lastButtonPtr;

unsigned int defaultActuator = ACTUATOR_LRA;		// Actuator used when no actuator is specified

static unsigned char BinaryCapTouch_mode;

/**
 * BinaryModes - function containing the extra binary mode button functions and effects
 */
void BinaryModes(void)
{
	if(CapTouch_GetBinaryMode() == BINARYMODESLEVEL)
	{

		BinaryCapTouch_mode = CapTouch_mode >> 3;

	switch(BinaryCapTouch_mode)
	{
		case 0:		// Mode 0 -  GUI Mode - Enable part for GUI control
		{
			static unsigned int trig = 0;			// Local trigger flag

			switch(buttonPtr->referenceNumber)
			{
				case BUTTON1:	// Set ERM mode
				{
					Haptics_Start(MODE_RAM);					// Enable DRV260x
					Haptics_SetActuator(ACTUATOR_ERM);			// Set actuator member variable to ERM
					Haptics_SetTrigger(TRIGGER_INTERNAL);		// Set trigger member variable to internal
					Haptics_HardwareMode(MODE_RAM);				// Set actuator load switch
					Haptics_SendTriggerType();					// Send trigger type to DRV260x
					Haptics_SendActuatorSettings();				// Send actuator settings to DRV260x

					defaultActuator = ACTUATOR_ERM; // Set the binary modes default actuator
					break;
				}
				case BUTTON2:	// Set LRA mode
				{
					Haptics_Start(MODE_RAM);					// Enable DRV260x
					Haptics_SetActuator(ACTUATOR_LRA);			// Set actuator member variable to LRA
					Haptics_SetTrigger(TRIGGER_INTERNAL);		// Set trigger member variable to internal
					Haptics_HardwareMode(MODE_RAM);				// Set actuator load switch
					Haptics_SendTriggerType();					// Send trigger type to DRV260x
					Haptics_SendActuatorSettings();				// Send actuator settings to DRV260x

					defaultActuator = ACTUATOR_LRA; // Set the binary modes default actuator
					break;
				}
				case BUTTON3:	// Trigger Select (Internal, External Edge, External Level)
				{
					trig++;			// Increment trigger

					if(trig > 2)	// If trig > 2, reset trigger type
						trig = 0;

					/* Show current trigger setting */
					CapTouch_ModeLEDsOff();
					sleep(1000);
					CapTouch_ModeLEDBinary(trig+1);
					sleep(5000);
					CapTouch_ModeLEDsOff();

					if(trig == TRIGGER_EXTERNAL_LEVEL)	// External Level Trigger
					{
						Haptics_EnableAmplifier();					// Enable DRV260x
						Haptics_SetTrigger(TRIGGER_EXTERNAL_LEVEL); // Set trigger member variable to external level
						Haptics_HardwareMode(0);					// Configure the hardware
						Haptics_SendTriggerType();					// Send trigger settings to DRV260x
					}
					else if(trig == TRIGGER_EXTERNAL_EDGE)	// External Edge Trigger
					{
						Haptics_EnableAmplifier();					// Enable DRV260x
						Haptics_SetTrigger(TRIGGER_EXTERNAL_EDGE); 	// Set trigger member variable to edge
						Haptics_HardwareMode(0);					// Configure the hardware
						Haptics_SendTriggerType();					// Send trigger settings to DRV260x
					}
					else	// Internal trigger
					{
						Haptics_EnableAmplifier();					// Enable DRV260x
						Haptics_SetTrigger(TRIGGER_INTERNAL); 		// Set trigger member variable to internal
						Haptics_HardwareMode(0);					// Configure the hardware
						Haptics_SendTriggerType();					// Send trigger settings to DRV260x
					}

					break;
				}
				case BUTTON4:	// Trigger button
				{
					Haptics_EnableAmplifier();					// Enable Amplifier

					if(I2C_ReadSingleByte(DRV260x_GO) == GO)	// If waveform is playing
					{
						Haptics_DisableTrigger(trig);			// Disable waveform based on trigger type
					}
					else										// Else trigger waveform to begin playing
					{
						Haptics_SendTriggerType();				// Set trigger type
						Haptics_EnableTrigger();				// Send trigger, select trigger with button 3
						Haptics_EffectActive(0);				// Disable effect active flag to prevent auto-shutdown of DRV260x
					}
					break;
				}
				default: __no_operation();
			}
		}
		break;
		case 1:		// Mode 1 - Auto-Calibration & Diagnostics
		{
			unsigned char diagnosticResults = 0;	// Stores status register after diagnostics

			switch(buttonPtr->referenceNumber)
			{
				case BUTTON1:	// ERM Auto-Calibration
				{
					Haptics_RecordAutoCalValues(1);
					Haptics_RunAutoCal_ERM();
					Haptics_EnableAmplifier();		// Re-enable amplifier for external communication

					defaultActuator = ACTUATOR_ERM; // Set the binary modes default actuator
					break;
				}
				case BUTTON2:	// LRA Auto-Calibration
				{
					Haptics_RecordAutoCalValues(1);
					Haptics_RunAutoCal_LRA();
					Haptics_EnableAmplifier();		// Re-enable amplifier for external communication

					defaultActuator = ACTUATOR_LRA; // Set the binary modes default actuator
					break;
				}
				case BUTTON3:	// ERM Diagnostics
				{
					diagnosticResults = Haptics_Diagnostics(ACTUATOR_ERM);

					/* Display Status Register[3:0] on mode LEDs */
					CapTouch_ModeLEDsOff();
					sleep(LEDBLINKDELAY);
					CapTouch_ModeLEDBinary(diagnosticResults & 0x0F);
					sleep(20000);
					CapTouch_ModeLEDsOff();
					sleep(LEDBLINKDELAY);
					break;
				}
				case BUTTON4:	// LRA Diagnostics
				{
					diagnosticResults = Haptics_Diagnostics(ACTUATOR_LRA);

					/* Display Status Register[3:0] on mode LEDs */
					CapTouch_ModeLEDsOff();
					sleep(LEDBLINKDELAY);
					CapTouch_ModeLEDBinary(diagnosticResults & 0x0F);
					sleep(20000);
					CapTouch_ModeLEDsOff();
					sleep(LEDBLINKDELAY);
					break;
				}
				default: __no_operation();
			}
		}
		break;
		case 2:		// Mode 2 - External PWM Input Source (using MSP430 to select actuator type)
		{
			CapTouch_ButtonLEDsOff();

			switch(buttonPtr->referenceNumber)
			{
				case BUTTON1:	// Disable Amplifier
				{
					Haptics_DisableAmplifier();
					P1OUT = BUTTON1;
					break;
				}
				case BUTTON2:	// ERM Mode
				{
					Haptics_Start(MODE_PWM);
					Haptics_SetActuator(ACTUATOR_ERM);
					Haptics_SetTrigger(TRIGGER_INTERNAL);
					Haptics_SendTriggerType();
					Haptics_SendActuatorSettings();
					I2C_WriteSingleByte(DRV260x_MODE, PWM_Analog);		// Set PWM input mode
					P1OUT = BUTTON2;
					break;
				}
				case BUTTON3:	// LRA Mode
				{
					Haptics_Start(MODE_PWM);
					Haptics_SetActuator(ACTUATOR_LRA);
					Haptics_SetTrigger(TRIGGER_INTERNAL);
					Haptics_HardwareMode(MODE_PWM);
					Haptics_SendTriggerType();
					Haptics_SendActuatorSettings();
					I2C_WriteSingleByte(DRV260x_MODE, PWM_Analog);		// Set PWM input mode
					P1OUT = BUTTON3;
					break;
				}
				case BUTTON4:
				{
					Haptics_DisableAmplifier();
					CapTouch_FlashModeLEDs(3);
					break;
				}
				default: __no_operation();
			}

			CapTouch_FreezeLEDs();
		}
		break;
		case 3:		// Mode 3 - External PWM and Enable Input Source (MSP430 EN Pin is set to Hi-Z)
		{
			CapTouch_ButtonLEDsOff();

			switch(buttonPtr->referenceNumber)
			{
				case BUTTON1:	// Return to normal operation (Must be pressed before changing modes)
				{
					Haptics_EnableAmplifier();
					I2C_WriteSingleByte(DRV260x_MODE, Int_Trig);	// Set PWM input mode
					Haptics_DisableAmplifier();
					P3DIR |= BIT1;
					P1OUT = BUTTON1;
					break;
				}
				case BUTTON2:	// ERM Mode with Hi-Z EN
				{
					Haptics_EnableAmplifier();
					I2C_WriteSingleByte(DRV260x_MODE, PWM_Analog);	// Set PWM input mode
					Haptics_SetActuator(ACTUATOR_ERM);
					Haptics_SendActuatorSettings();
					Haptics_DisableAmplifier();
					P3DIR &= ~(BIT1);								// Hi-Z EN pin on MSP430
					Haptics_LoadSwitchSelect(ACTUATOR_ERM);
					P1OUT = BUTTON2;
					break;
				}
				case BUTTON3:	// LRA Mode with Hi-Z EN
				{
					Haptics_EnableAmplifier();
					I2C_WriteSingleByte(DRV260x_MODE, PWM_Analog);	// Set PWM input mode
					Haptics_SetActuator(ACTUATOR_LRA);
					Haptics_SendActuatorSettings();
					Haptics_DisableAmplifier();
					P3DIR &= ~(BIT1);								// Hi-Z EN pin on MSP430
					Haptics_LoadSwitchSelect(ACTUATOR_LRA);
					P1OUT = BUTTON3;
					break;
				}
				case BUTTON4:
				{
					Haptics_EnableAmplifier();
					I2C_WriteSingleByte(DRV260x_MODE, Int_Trig);	// Set PWM input mode
					Haptics_DisableAmplifier();
					P3DIR |= BIT1;
					CapTouch_FlashModeLEDs(3);
					break;
				}
				default: __no_operation();
			}
			CapTouch_FreezeLEDs();
		}
		break;
		case 4:		// Mode 4 - Analog Input Mode
		{
			CapTouch_ButtonLEDsOff();

			static uint8_t IsAnalogEnable = 0;

			switch(buttonPtr->referenceNumber)
			{
				case BUTTON1:	// Analog AC - ERM
				{
					if(IsAnalogEnable)
					{
						P1OUT = 0;
						Haptics_Stop(MODE_ANALOG);
						IsAnalogEnable = 0;
					}
					else
					{
						P1OUT = BUTTON1;
						Haptics_SendWaveform(Analog_AC, ACTUATOR_ERM, TRIGGER_INTERNAL);
						Haptics_EffectActive(0);
						IsAnalogEnable = 1;
					}
					break;
				}
				case BUTTON2:	// Analog DC - ERM
				{
					if(IsAnalogEnable)
					{
						P1OUT = 0;
						Haptics_Stop(MODE_ANALOG);
						IsAnalogEnable = 0;
					}
					else
					{
						P1OUT = BUTTON2;
						Haptics_SendWaveform(Analog_DC, ACTUATOR_ERM, TRIGGER_INTERNAL);
						Haptics_EffectActive(0);
						IsAnalogEnable = 1;
					}
					break;
				}
				case BUTTON3:	// Analog AC - LRA
				{
					if(IsAnalogEnable)
					{
						P1OUT = 0;
						Haptics_Stop(MODE_ANALOG);
						IsAnalogEnable = 0;
					}
					else
					{
						P1OUT = BUTTON3;
						Haptics_SendWaveform(Analog_AC, ACTUATOR_LRA, TRIGGER_INTERNAL);
						Haptics_EffectActive(0);
						IsAnalogEnable = 1;
					}
					break;
				}
				case BUTTON4:	// Analog DC - LRA
				{
					if(IsAnalogEnable)
					{
						P1OUT = 0;
						Haptics_Stop(MODE_ANALOG);
						IsAnalogEnable = 0;
					}
					else
					{
						P1OUT = BUTTON4;
						Haptics_SendWaveform(Analog_DC, ACTUATOR_LRA, TRIGGER_INTERNAL);
						Haptics_EffectActive(0);
						IsAnalogEnable = 1;
					}
					break;
				}
				default: __no_operation();
			}

			CapTouch_FreezeLEDs();
		}
		break;
		case 5:		// Mode 5 - Auto-resonance Off (Open Loop) Frequency Adjust mode
		{
			static uint16_t currentFrequency = 0x151;
			static uint8_t j = 0;

			switch(buttonPtr->referenceNumber)
			{
				case BUTTON1:		// Auto-resonance ON (Closed Loop) Alert
				{
					Haptics_SetLRAOpenLoopMode(0);										// Set auto-resonance on
					Haptics_SendWaveform(RTP_Alert, ACTUATOR_LRA, TRIGGER_INTERNAL);	// Play closed-loop waveform
					break;
				}
				case BUTTON2:		// Auto-resonance OFF (Open Loop) Alert
				{
					TA1CCR0 = currentFrequency;								// Set PWM frequency
					Haptics_SetLRAOpenLoopMode(1);							// Set open loop mode

					/* Output Waveform */
					Haptics_Start(MODE_PWM);								// Enable the amplifier, begin PWM (if required)
					Haptics_HardwareMode(MODE_PWM);							// Configure the Hardware (load switch, GPIOs)
					Haptics_SendTriggerType();								// Send trigger type to DRV260x
					Haptics_SendActuatorSettings();							// Send actuator settings to DRV260x
					Haptics_DisableTriggerActive();							// Stop waveform playback
					I2C_WriteSingleByte(DRV260x_MODE, PWM_Analog);			// Set PWM input mode
					TA1CCR1 = currentFrequency;								// Set output to 100% output duty cycle
					for(j=0; j < 0xA0; ++j)									// Count number of 5ms intervals
					{
						timerdelay(TICKDELAY);								// Time delay before next change in amplitude
					}

					Haptics_Stop(MODE_PWM);									// Stop PWM

					TA1CCR0 = (unsigned int) LRAFREQ;  						// Reset default frequency
					break;
				}
				case BUTTON3:		// Decrease open-loop frequency
				{
					if(currentFrequency < 0x1AF)							// If greater than minimum frequency
					{
						if(Haptics_IsOutputEnabled())
						{
							currentFrequency = currentFrequency + OPENLOOPFREQSTEP;
							TA1CCR0 = currentFrequency;							// Set PWM frequency
							Haptics_SetLRAOpenLoopMode(1);						// Set open loop mode

							/* Output Waveform */
							Haptics_Start(MODE_PWM);							// Enable the amplifier, begin PWM (if required)
							Haptics_HardwareMode(MODE_PWM);						// Configure the Hardware (load switch, GPIOs)
							Haptics_SendTriggerType();							// Send trigger type to DRV260x
							Haptics_SendActuatorSettings();						// Send actuator settings to DRV260x
							Haptics_DisableTriggerActive();						// Stop waveform playback
							I2C_WriteSingleByte(DRV260x_MODE, PWM_Analog);		// Set PWM input mode
							TA1CCR1 = currentFrequency;							// Set output to 100% output duty cycle
							for(j=0; j < 0x80; ++j)								// Count number of 5ms intervals
							{
								timerdelay(TICKDELAY);							// Time delay before next change in amplitude
							}
							Haptics_Stop(MODE_PWM);								// Stop PWM

							TA1CCR0 = (unsigned int) LRAFREQ;  					// Reset default frequency
							TA1CCR1 = LRAFREQ / 2;

							sleep(500);
						}
					}
					break;
				}
				case BUTTON4:		// Increase open-loop frequency
				{
					if(currentFrequency > 0x104)							// If less than maximum frequency
					{
						if(Haptics_IsOutputEnabled())
						{
							currentFrequency = currentFrequency - OPENLOOPFREQSTEP;
							TA1CCR0 = currentFrequency;							// Set PWM frequency

							Haptics_SetLRAOpenLoopMode(1);						// Set open loop mode

							/* Output Waveform */
							Haptics_Start(MODE_PWM);							// Enable the amplifier, begin PWM (if required)
							Haptics_HardwareMode(MODE_PWM);						// Configure the Hardware (load switch, GPIOs)
							Haptics_SendTriggerType();							// Send trigger type to DRV260x
							Haptics_SendActuatorSettings();						// Send actuator settings to DRV260x
							Haptics_DisableTriggerActive();						// Stop waveform playback
							I2C_WriteSingleByte(DRV260x_MODE, PWM_Analog);		// Set PWM input mode
							TA1CCR1 = currentFrequency;							// Set output to 100% output duty cycle
							for(j=0; j < 0x80; ++j)								// Count number of 5ms intervals
							{
								timerdelay(TICKDELAY);							// Time delay before next change in amplitude
							}
							Haptics_Stop(MODE_PWM);								// Stop PWM

							TA1CCR0 = (unsigned int) LRAFREQ;  					// Reset default frequency
							TA1CCR1 = LRAFREQ / 2;

							sleep(500);
						}
					}
					break;
				}
				default: __no_operation();
			}

			CapTouch_RepeatOnHoldWait();
		}
		break;
		case 6:		// Mode 6 - Life Test (RTP) - 2s ON - 1s OFF
		{
			static signed char amplitude = 0x7F;

			/* Test Click waveform data */
			unsigned char RTP_testclick_data[] = {
					amplitude, 0x05,
					0x00, 0x07
			};
			const Waveform RTP_testclick = {MODE_RTP,4,RTP_testclick_data};

			Haptics_SetERMOpenLoopMode(0);						// Set ERM closed-loop mode

			switch(buttonPtr->referenceNumber)
			{
				case BUTTON1:	// 2 seconds ON, 1 second OFF
				{
					/* Life Test waveform data */
					unsigned char RTP_onofflifetest_data[] = {
							amplitude, 0xFF,
							amplitude, 0x90,
							0x00, 0xC8
					};
					const Waveform RTP_onoff = {MODE_RTP,6,RTP_onofflifetest_data};

					Haptics_SetActuator(defaultActuator); 		// Select the actuator type
					Haptics_SetTrigger(TRIGGER_INTERNAL);		// Select the trigger type
					Haptics_Start(RTP_onoff.inputMode);			// Enable the amplifier, begin PWM
					Haptics_HardwareMode(RTP_onoff.inputMode);	// Configure the Hardware
					Haptics_SendTriggerType();					// Send trigger type to DRV260x
					Haptics_SendActuatorSettings();				// Send actuator settings to DRV260x


					while(1)
					{
						Haptics_OutputWaveform(RTP_onoff);		// Send waveform data to driver
					}
				}
				case BUTTON2:	// Test Buzz
				{
					unsigned char RTP_on_data[] = {
							amplitude, 0xFF,
							0x80, 0x00
					};
					const Waveform RTP_on = {MODE_RTP,4,RTP_on_data};

					Haptics_SetActuator(defaultActuator); 			// Select the actuator type
					Haptics_SendWaveformDefault(RTP_on);

					break;
				}
				case BUTTON3:	// Decrement Amplitude
				{
					if(amplitude > 1)
					{
						amplitude -= 1;
						Haptics_SetActuator(defaultActuator); 		// Select the actuator type
						Haptics_SendWaveformDefault(RTP_testclick);
					}
					break;
				}
				case BUTTON4:	// Increment Amplitude
				{
					if(amplitude < 0x7F)
					{
						amplitude += 1;
						Haptics_SetActuator(defaultActuator); 		// Select the actuator type
						Haptics_SendWaveformDefault(RTP_testclick);
					}
					break;
				}
				default: __no_operation();
			}
		}
		break;
		case 7:		// Mode 7 - Life Test (RTP) - Infinite Buzz
		{
			static signed char 	amplitude = 0x7F;

			/* Test Click waveform data */
			unsigned char RTP_testclick_data[] = {
					amplitude, 0x05,
					0x00, 0x07
			};
			const Waveform RTP_testclick = {MODE_RTP,4,RTP_testclick_data};

			Haptics_SetERMOpenLoopMode(0);						// Set ERM closed-loop mode

			switch(buttonPtr->referenceNumber)
			{
				case BUTTON1:	// Infinite Buzz
				{
					/* Life Test waveform data */
					unsigned char RTP_infinitelifetest_data[] = {
							amplitude, 0x01
					};
					const Waveform RTP_infinitelifetest = {MODE_RTP,2,RTP_infinitelifetest_data};

					Haptics_SetActuator(defaultActuator); 					// Select the actuator type
					Haptics_SetTrigger(TRIGGER_INTERNAL);					// Select the trigger type
					Haptics_EnableAmplifier();								// Enable Amplifier
					Haptics_HardwareMode(RTP_infinitelifetest.inputMode);	// Configure the Hardware
					Haptics_SendTriggerType();								// Send trigger type to DRV260x
					Haptics_SendActuatorSettings();							// Send actuator settings to DRV260x
					Haptics_OutputWaveform(RTP_infinitelifetest);			// Send waveform data to driver

					while(1);
				}
				case BUTTON2:	// Test Buzz
				{
					unsigned char RTP_on_data[] = {
							amplitude, 0xFF,
							0x80, 0x00
					};
					const Waveform RTP_on = {MODE_RTP,4,RTP_on_data};

					Haptics_SetActuator(defaultActuator); 			// Select the actuator type
					Haptics_SendWaveformDefault(RTP_on);

					break;
				}
				case BUTTON3:	// Decrement Amplitude
				{
					if(amplitude > 1)
					{
						amplitude -= 1;
						Haptics_SetActuator(defaultActuator); 		// Select the actuator type
						Haptics_SendWaveformDefault(RTP_testclick);
					}
					break;
				}
				case BUTTON4:	// Increment Amplitude
				{
					if(amplitude < 0x7F)
					{
						amplitude += 1;
						Haptics_SetActuator(defaultActuator); 		// Select the actuator type
						Haptics_SendWaveformDefault(RTP_testclick);
					}
					break;
				}
				default: __no_operation();
			}
		}
		break;
		case 8:		// Mode 8 - Life Test (PWM) - 2s ON, 1s OFF
		{
			static uint8_t amplitude = 0xFA;

			unsigned char lra_testclick_data[] = {
					amplitude, 0x05,
					0x00, 0x07
			};
			const Waveform lra_testclick = {MODE_PWM,4,lra_testclick_data};

			Haptics_SetERMOpenLoopMode(0);					// Set ERM closed-loop mode

			switch(buttonPtr->referenceNumber)
			{
			case BUTTON1:	// Begin Life Test
			{
				unsigned char lra_onofflifetest_data[] = {
						amplitude, 0xFF,
						amplitude, 0x90,
						0x80, 0xC8
				};
				const Waveform lra_onofflifetest = {MODE_PWM,6,lra_onofflifetest_data};

				Haptics_SetActuator(defaultActuator); 		// Select the actuator type

				while(1)
				{
					Haptics_SetERMOpenLoopMode(0);			// Set ERM closed-loop mode
					Haptics_SendWaveformDefault(lra_onofflifetest);
				}
			}
			case BUTTON2:	// Test buzz, used to determine output Vrms
			{
				unsigned char lra_on_data[] = {
						amplitude, 0xFF,
						0x80, 0x00
				};
				const Waveform lra_on = {MODE_PWM,4,lra_on_data};

				Haptics_SetActuator(defaultActuator); 			// Select the actuator type
				Haptics_SendWaveformDefault(lra_on);
				break;
			}
			case BUTTON3:	// Decrement amplitude
			{
				if(amplitude > 1)
				{
					amplitude -= 1;
					Haptics_SetActuator(defaultActuator); 		// Select the actuator type
					Haptics_SendWaveformDefault(lra_testclick);
				}
				break;
			}
			case BUTTON4:	// Increment amplitude
			{
				if(amplitude < 255)
				{
					amplitude += 1;
					Haptics_SetActuator(defaultActuator); 		// Select the actuator type
					Haptics_SendWaveformDefault(lra_testclick);
				}
				break;
			}
			default: __no_operation();
			}
		}
		break;
		case 9:		// Mode 9 - Recorder
		{
			CapTouch_ButtonLEDsOff();

			switch(buttonPtr->referenceNumber)
			{
			case BUTTON1:
			{
				#ifdef RECORDER_H_			// If Recorder.h is #included
				Recorder_RecordToggle();	// Press to begin recording
				#endif
				break;

			}
			case BUTTON2:
			{
				// After pressing button 1, press this button to create a pattern
				break;
			}
			case BUTTON3:
			{
				#ifdef RECORDER_H_			// If Recorder.h is #included
				Recorder_PlayToggle();		// Press to play/stop patter
				#endif
				break;
			}
			case BUTTON4:
			{
				break;
			}
			default: __no_operation();
			}
		}
		break;
		case 30:	// Mode 30 - Actuator Break-In
		{
			/* Test Click waveform data */
			unsigned char RTP_testclick_data[] = {
					0x3F, 0x05,
					0x00, 0x05
			};
			const Waveform RTP_testclick = {MODE_RTP,4,RTP_testclick_data};

			/* Life Test waveform data */
			unsigned char RTP_onofflifetest_data[] = {
					0x3F, 0xFF,
					0x3F, 0xFF,
					0x3F, 0xFF
			};
			const Waveform RTP_onoff = {MODE_RTP,6,RTP_onofflifetest_data};


			switch(buttonPtr->referenceNumber)
			{
				case BUTTON1:
				{
					unsigned char i = 0;

					Haptics_SetActuator(ACTUATOR_LRA);			// Select the actuator type
					Haptics_SetTrigger(TRIGGER_INTERNAL);		// Select the trigger type
					Haptics_Start(RTP_onoff.inputMode);			// Enable the amplifier, begin PWM
					Haptics_HardwareMode(RTP_onoff.inputMode);	// Configure the Hardware
					Haptics_SendTriggerType();					// Send trigger type to DRV260x
					Haptics_SendActuatorSettings();				// Send actuator settings to DRV260x

					for(i = 0; i < 8; i++)
					{
						Haptics_OutputWaveform(RTP_onoff);			// Send waveform data to driver

						Haptics_OutputWaveform(RTP_testclick);
						Haptics_OutputWaveform(RTP_testclick);
						Haptics_OutputWaveform(RTP_testclick);
						Haptics_OutputWaveform(RTP_testclick);
						Haptics_OutputWaveform(RTP_testclick);

						if(Haptics_IsActuatorLRA())
							Haptics_SetActuator(ACTUATOR_ERM);
						else
							Haptics_SetActuator(ACTUATOR_LRA);

						Haptics_SendActuatorSettings();
					}

					Haptics_Stop(MODE_RTP);

					break;
				}
				case BUTTON2:
				{

					break;
				}
				case BUTTON3:
				{

					break;
				}
				case BUTTON4:
				{

					break;
				}
				default: __no_operation();
			}
		}
		break;
		case 31:	// Mode 31 (All LEDs On) - Silicon Revision, Code Revision Flash
		{
			switch(buttonPtr->referenceNumber)
			{
				case BUTTON1:
				{
					CapTouch_FlashDeviceID();		// Flash device ID
					break;
				}
				case BUTTON2:
				{
					CapTouch_FlashSiRevision();		// Flash Silicon Revision (Should be "1")
					break;
				}
				case BUTTON3:
				{
					CapTouch_FlashCodeRevision();	// Flash Software Code Revision
					break;
				}
				case BUTTON4:
				{

					break;
				}
				default: __no_operation();
			}
		}	// End Mode 31

	};	/* End switch(CapTouch_mode) */

	}
}

/**
 * BinaryModesRelease - contains the button release functions for extra binary modes
 */
void BinaryModesRelease()
{
	if(CapTouch_GetBinaryMode() == BINARYMODESLEVEL)
	{
		BinaryCapTouch_mode = CapTouch_mode >> 3;

		switch(CapTouch_mode)
		{
			case 0:		// Mode 0 - GUI Mode - Enable part for GUI control
			{
				switch(lastButtonPtr->referenceNumber)
				{
					case BUTTON1:
					{

						break;
					}
					case BUTTON2:
					{

						break;
					}
					case BUTTON3:
					{

						break;
					}
					case BUTTON4:	// Trigger Button Release
					{
						Haptics_DisableTrigger(TRIGGER_EXTERNAL_LEVEL);
						break;
					}
					default: __no_operation();
				}
			}
			break;
		}
	}
}


#endif /* EXTRAMODES_C_ */
