/******************************************************************************
 * LibraryModes.h
 *
 * Created on: July 28, 2012
 * Board: DRV2605EVM-CT RevA
 * Author: a0866685
 *
 * Desc: This file contains the button functions to access the DRV2605 ROM .
 * 		library.  Include this file to compile with the DRV2605 ROM library
 * 		modes.
 *
 * 		Mode 0 - 30 are the library effects and Mode 31 allows the user to
 * 		change ROM libraries.
 *
 ******************************************************************************/

/* *****************************************************************************
 * Button Descriptions
 * *****************************************************************************
 * To Access:  Hold the "+" button for approximately 3 seconds, twice.  The
 * 		mode LEDs will flash.
 *
 * The mode LEDs will count in binary.
 *
 * Mode 0 (All LEDs Off):
 * B1: Library Effect 1 - StrongClick_100
 * B2: Library Effect 2 - StrongClick_60
 * B3: Library Effect 3 - StrongClick_30
 * B4: Library Effect 4 - SharpClick_100
 *
 * Mode 1 (M0 On):
 * B1: Library Effect 5
 * B2: Library Effect 6
 * B3: Library Effect 7
 * B4: Library Effect 8
 *
 * Mode 2 (M1 On):
 * ...
 *
 * Mode 3 (M1 and M0 On):
 * ...
 *
 * Mode 31
 * B1: Select ERM Library 1-5
 * B2: Select LRA Library 6
 * B3: Empty
 * B4: Empty
 *
 *******************************************************************************/

#ifndef LIBRARYMODES_C_
#define LIBRARYMODES_C_

#include "LibraryModes.h"
#include "DRV2605.h"
#include "Actuator_Waveforms.h"
#include "CapTouchBoard.h"
#include "I2C.h"

extern struct Element* modePtr;
extern struct Element* buttonPtr;

/**
 * LibraryModes - function containing the DRV2605 ROM library modes where BIT0 = 1 (ie = xxxxx001'b)
 */
void LibraryModes(void)
{
	static uint8_t library = 6;							// Default to LRA library
	static uint8_t actuatorType = ACTUATOR_LRA;			// Select LRA actuator
	uint8_t libraryMode;								// Current set of buttons

	// To access the DRV2605 ROM library hold the "+" button
	if(CapTouch_GetBinaryMode() == LIBRARYMODESLEVEL)
	{
		libraryMode = (CapTouch_mode >> 1);				// CapTouch_Mode[7:3] >> 3 * 4

		Haptics_SetActuator(actuatorType);				// Set actuator type
		Haptics_SetLibrary(library);					// Set library
		Haptics_SetTrigger(TRIGGER_INTERNAL);			// Set internal trigger

		if(libraryMode == 124)
		{
			switch(buttonPtr->referenceNumber)
			{
				case BUTTON1:
				{
					library++;							// Increment ROM Library Selection

					if(library > 5)						// If last ERM library, go to 1
						library = 1;

					actuatorType = ACTUATOR_ERM;		// Set actuator type to ERM

					CapTouch_ModeLEDsOff();
					sleep(1000);
					CapTouch_ModeLEDBinary(library);	// Show current library number
					sleep(5000);
					CapTouch_ModeLEDsOff();

					break;
				}
				case BUTTON2:
				{
					library = 6;						// Set ROM Library to LRA (6)
					actuatorType = ACTUATOR_LRA;		// Set actuator type to LRA

					CapTouch_ModeLEDsOff();
					sleep(1000);
					CapTouch_ModeLEDBinary(library);	// Show current library number
					sleep(5000);
					CapTouch_ModeLEDsOff();
					break;
				}
				case BUTTON3:
				{
					break;
				}
				case BUTTON4:
				{
					break;
				}
			}
		}
		else	// Otherwise play effects
		{
			switch(buttonPtr->referenceNumber)
			{
				case BUTTON1:
				{
					Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ1, (libraryMode + 1));
					Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ2, 0x00);
					Haptics_SendWaveformDefault(LRA_ROM);
					break;
				}
				case BUTTON2:
				{
					Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ1, (libraryMode + 2));
					Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ2, 0x00);
					Haptics_SendWaveformDefault(LRA_ROM);

					break;
				}
				case BUTTON3:
				{
					Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ1, (libraryMode + 3));
					Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ2, 0x00);
					Haptics_SendWaveformDefault(LRA_ROM);
					break;
				}
				case BUTTON4:
				{
					Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ1, (libraryMode + 4));
					Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ2, 0x00);
					Haptics_SendWaveformDefault(LRA_ROM);
					break;
				}

			}
		}
	}
}


#endif /* LIBRARYMODES_C_ */
