/*******************************************************************************
 * main.c
 *
 * 	Created on : April 23, 2012
 * 	      Board: DRV2605EVM-CT REVA
 *
 * Description: This program controls the DRV2605EVK-CT evaluation board.
 * 
 * I/O
 * 		   -------------
 * 		  |			P1.0|--> Red LED
 * 		  |			P1.1|--> Green LED
 * 		  |			P1.2|--> Yellow LED
 *	M0 <--|P3.3		P1.3|--> Blue LED
 * 	M1 <--|P3.4			|
 * 	M2 <--|P3.5		P2.0|<-- B1 Button
 * 	M3 <--|P3.6		P2.1|<-- B2 Button
 * 	M4 <--|P3.7  	P2.2|<-- B3 Button
 * 		  |			P2.3|<-- B4 Button
 * 		  |			P2.4|<-- Minus Button
 * SCL <->|P1.6		P2.5|<-- Plus button
 * SDA <->|P1.7		P2.6|--> Load Switch (ERM or LRA)
 * 		  |			P3.1|--> Enable
 * 		  |			P3.2|--> IN/TRIG
 *		   -------------
 *
 * Last Updated: December 27, 2012
 * *****************************************************************************
 * Button Descriptions 
 * *****************************************************************************
 * Mode OFF (All LEDs Off) - ERM & LRA Effects
 * B1: ERM Click + Ramp Down
 * B2: ERM Short Ramp Up + Pulsing
 * B3: LRA Click + Ramp Down
 * B4: LRA Short Ramp Up + Pulsing
 *
 * Mode 4 (M4 On) -	ERM Clicks
 * B1: Sharp Click 100%
 * B2: Strong Click 100% + Release
 * B3: Soft Bump 100%
 * B4: Double Click 100%
 *
 * Mode 3 (M3 On) - LRA Clicks
 * B1: Sharp Click 100%
 * B2: Strong Click 100% + Release
 * B3: Soft Bump 100%
 * B4: Double Click 100%
 * 
 * Mode 2 (M2 On)
 * B1: LRA Auto-resonance ON (Closed-loop)
 * B2: LRA Auto-resonance OFF (Open-Loop)
 * B3: ERM Buzz Alert
 * B4:
 * 
 * Mode 1 (M1 On) - Concentration / Simon Game
 * B1: Start Game
 * B2: Start Game
 * B3: Start Game
 * B4: Start Game
 *
 * Mode 0 (M1 On)
 * B1: ERM Audio-to-Haptics (A2H) Enable/Disable
 * B2: LRA Audio-to-Haptics (A2H) Enable/Disable
 * B3: While in A2H, switch to internal trigger, play click and then switch back to A2H
 * B4: While in A2H, switch to internal trigger, play buzz and then switch back to A2H
 *
 * ------------------------- Note ------------------------------------------------------
 * Instructions for excluding certain functionality to decrease build size
 * 1) Comment SimonGame.h and/or Recorder.h
 * 2) Right click on SimonGame.c and/or Recorder.c in Project Explorer
 * 3) Select Resource Configurations
 * 4) Select Exclude from Build
 * 5) Select the debug and release check boxes and press OK
 *
 * To include the SimonGame.h and/or Recorder.h functionality do the reverse.
 ***************************************************************************************/

#include "msp430g2553.h"
#include "CapTouchBoard.h"
#include "Actuator_Waveforms.h"
#include "SimonGame.h"
#include "Timer.h"
#include "I2C.h"
#include "Haptics.h"
#include "BinaryModes.h"
#include "LibraryModes.h"
#include "DRV2605.h"
#include "BinaryModes.h"
#include "Recorder.h"

/* Capacitive Touch Button Pointers */
struct Element* modePtr;
struct Element* buttonPtr;
struct Element* lastButtonPtr;

/* Main Variables */
static unsigned char a2h_setting = 0;		// Indicate current A2H mode (do not edit)

const uint8_t buttonLeds[NUMBER_BUTTONS] =
{
  BIT0,
  BIT1,
  BIT2,
  BIT3
};

const struct Element* buttonList[NUMBER_BUTTONS] = 
{
  &button1,
  &button2,
  &button3,
  &button4
};            

unsigned int i;

// Main Function
void main(void)
{ 
  WDTCTL = WDTPW + WDTHOLD;             // Stop watchdog timer
  
  // Initialize MSP430 Clocks
  BCSCTL1 = CALBC1_8MHZ;                // Set DCO to 1, 8, 12 or 16MHz
  DCOCTL = CALDCO_8MHZ;					// Set MODx to 3
  BCSCTL1 |= DIVA_0;                    // ACLK/(0:1,1:2,2:4,3:8)
  BCSCTL2 |= DIVS_0;					// SMCLK/(0=1,1=2,2=4,3=8)
  BCSCTL3 |= LFXT1S_2;                  // LFXT1 = VLO
  
  // Set GPIO functions
  P2SEL &= 0x3F;
  P2SEL |= 0x3F;						// P2.0-P2.5 = cap sense, P2.6-P2.7 = I/O
  P2SEL2 |= 0x3F;						// P2.0-P2.5 = cap sense, P2.6-P2.7 = I/O
  
  // Set GPIO directions
  P1DIR |= (BUTTON1+BUTTON2+BUTTON3+BUTTON4);			// Set P1.0-P1.3 to outputs for colored LEDs
  P2DIR |= 0xC0;										// Set P2.6-P2.7 = Output
  P3DIR |= (MODE0+MODE1+MODE2+MODE3+MODE4+BIT1+BIT0);  	// Set P3.1 = EN LED (Output), P3.2 = IN/TRIG (Output), P3.3-P3.7 = MODE0-MODE4 LEDs (Output)
  
  // Set GPIO values
  P1OUT = 0x00;							// Colored LEDs off
  P3OUT = 0x00;                    		// White LEDs off, EN = off
 
  // TimerA Setup
  // Set PWM Frequency
  TA1CCR0 = 0x0151;   // 23.04 kHz
  
  CapTouch_Init();						// Initialize the cap touch board setting and statuses
  CapTouch_PowerUpSequence();			// CapTouch Board PowerUp Sequence
  Haptics_Init();						// Initialize Haptics settings
  Haptics_RunAutoCal_ERM();				// Run ERM Auto-Calibration
  Haptics_RunAutoCal_LRA();				// Run LRA Auto-Calibration

  #ifdef RECORDER_H_
  	Recorder_Init();						// Initialize Recorder
  #endif
  
  // Main loop starts here
  while (1)
  {
	/* Detect Button Presses */
    modePtr = (struct Element *)TI_CAPT_Buttons(&mode_change);		// detect mode button press
    buttonPtr = (struct Element *)TI_CAPT_Buttons(&buttons);		// detect effect button press

    // Binary Modes Sequence Recorder
	#ifdef RECORDER_H_												// If Recorder.h is #included
		Recorder_Record();												// Recorder button press sequence recorder
		Recorder_Playback();											// Recorder play back
	#endif

    /* Adjust Mode if mode button is pressed */
    if(modePtr)
    {

		CapTouch_UnfreezeLEDs(); 			// If color LEDs are frozen, unfreeze

		#ifdef SIMONGAME_H_
			Simon_Reset(); 					// If the mode button is pressed, cancel current Simon game
		#endif

		#ifdef RECORDER_H_					// If Recorder.h is #included
			Recorder_Reset();				// Reset recorder
		#endif

		if(CapTouch_isModeBtnReleased)		// if this is first time to register a mode button press
    	{
			Haptics_SetActuator(ACTUATOR_LRA);
			Haptics_SetTrigger(TRIGGER_INTERNAL);

    		CapTouch_isModeBtnReleased = 0; // Do not increment mode again until mode button is released and then next press again

    		if(modePtr == &mode_up)			// '+' button pressed
    		{
    			CapTouch_IncrementMode();	// increment mode
    			CapTouch_EnableOnHold(CapTouch_SetBinaryMode);			// change mode counter style to binary
    		}
    		else							// '-' button pressed
    		{
    			CapTouch_DecrementMode();	// decrement mode
    			CapTouch_EnableOnHold(CapTouch_UnsetBinaryMode);		// change mode counter style to carousel
    		}

    		if(CapTouch_mode == MODE0 && CapTouch_GetBinaryMode() == 0)	// if MODE0, setup audio-to-haptics
    		{
				a2h_setting = 0;
    			CapTouch_FreezeLEDs();		// Freeze color LEDs to represent Mode
    		}
    	}

		CapTouch_ModeLEDsOff();				// Clear the mode button LEDs
    }
    else		// else no mode button was pressed
    {
    	CapTouch_ModeRepeatReset();			// reset status and counters when repeat mode is finished
    	CapTouch_isModeBtnReleased = 1;		// Mode button has been released
    }

    P3OUT |= CapTouch_mode;					// Set mode LEDs to current mode

    // Manage the colored LEDs
    if(!CapTouch_isLEDFrozen)				// if LEDs are not frozen
    	CapTouch_ButtonLEDsOff();


    if(buttonPtr)		// if an effect button is pressed
    {
    	if(!CapTouch_isLEDFrozen)
    		P1OUT |= buttonPtr->referenceNumber;	// Turn on a colored LED for button B1-B4 pressed

		lastButtonPtr = buttonPtr;					// Store the button pressed as the lastButton

		if(CapTouch_isEffectBtnReleased)			// If this is the first time the button press was registered
		{
			CapTouch_isEffectBtnReleased = 0;		// Disable repeat effect (wait for button release)

			if(CapTouch_GetBinaryMode() == 0)		// Check if binary mode is disable
			{
				if(CapTouch_mode == MODEOFF)		// ERM & LRA Alerts
				{
					switch(buttonPtr->referenceNumber)
					{
					case BUTTON1:	// ERM Click + Ramp Down
					{
						Haptics_SetLibrary(ROM_A);
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ1, StrongClick_100);
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ2, TransitionRampDownMediumSmooth1_100to0);
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ3, 0x00);
						Haptics_SendWaveform(ERM_ROM, ACTUATOR_ERM, TRIGGER_INTERNAL);
						break;
					}
					case BUTTON2:	// ERM Short Ramp Up + Pulsing
					{
						Haptics_SetLibrary(ROM_A);
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ1, TransitionRampUpShortSharp1_0to100); //TransitionRampUpShortSmooth1_0to100);
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ2, PulsingStrong1_100);
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ3, 0x00);
						Haptics_SendWaveform(ERM_ROM, ACTUATOR_ERM, TRIGGER_INTERNAL);
						break;
					}
					case BUTTON3:	// LRA Click + Ramp Down
					{
						Haptics_SetLibrary(ROM_LRA);
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ1, StrongClick_100);
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ2, TransitionRampDownMediumSmooth1_100to0);
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ3, 0x00);
						Haptics_SendWaveform(LRA_ROM, ACTUATOR_LRA, TRIGGER_INTERNAL);
						break;
					}
					case BUTTON4:	// LRA Short Ramp Up + Pulsing
					{
						Haptics_SetLibrary(ROM_LRA);
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ1, TransitionRampUpShortSmooth1_0to100);
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ2, PulsingStrong1_100);
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ3, 0x00);
						Haptics_SendWaveform(LRA_ROM, ACTUATOR_LRA, TRIGGER_INTERNAL);
						break;
					}
					default: __no_operation();
					}

					CapTouch_RepeatOnHoldWait();
				}
				else if(CapTouch_mode == MODE4)		// Clicks ERM
				{
					Haptics_SetLibrary(ROM_A);
					switch(buttonPtr->referenceNumber)
					{
					case BUTTON1:	// ERM Sharp Click 100%
					{
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ1, SharpClick_100);
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ2, 0x00);
						Haptics_SendWaveform(ERM_ROM, ACTUATOR_ERM, TRIGGER_INTERNAL);
						CapTouch_RepeatOnHoldWait();
						break;
					}
					case BUTTON2:	// ERM Strong Click 60% + Release
					{
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ1, StrongClick_60);
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ2, 0x00);
						Haptics_SendWaveform(ERM_ROM, ACTUATOR_ERM, TRIGGER_EXTERNAL_EDGE);
						break;
					}
					case BUTTON3:	// ERM Soft Bump 100%
					{
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ1, SoftBump_100);
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ2, 0x00);
						Haptics_SendWaveform(ERM_ROM, ACTUATOR_ERM, TRIGGER_INTERNAL);
						CapTouch_RepeatOnHoldWait();
						break;
					}
					case BUTTON4:	// ERM Double Click 100%
					{
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ1, DoubleClick_100);
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ2, 0x00);
						Haptics_SendWaveform(ERM_ROM, ACTUATOR_ERM, TRIGGER_EXTERNAL_LEVEL);
						CapTouch_RepeatOnHoldWait();
						break;
					}
					default: __no_operation();
					}

				}
				else if(CapTouch_mode == MODE3)			//Clicks LRA
				{
					Haptics_SetLibrary(ROM_LRA);
					switch(buttonPtr->referenceNumber)
					{
					case BUTTON1:	// LRA Sharp Click 100%
					{
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ1, SharpTick2_80);
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ2, 0x00);
						Haptics_SendWaveform(LRA_ROM, ACTUATOR_LRA, TRIGGER_INTERNAL);
						CapTouch_RepeatOnHoldWait();
						break;
					}
					case BUTTON2:	// LRA Strong Click 100% + Release
					{
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ1, StrongClick_100);
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ2, 0x00);
						Haptics_SendWaveform(LRA_ROM, ACTUATOR_LRA, TRIGGER_EXTERNAL_EDGE);
						break;

					}
					case BUTTON3:	// LRA Soft Bump 100%
					{
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ1, SoftBump_100);
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ2, 0x00);
						Haptics_SendWaveform(LRA_ROM, ACTUATOR_LRA, TRIGGER_INTERNAL);
						CapTouch_RepeatOnHoldWait();
						break;
					}
					case BUTTON4:	// LRA Double Click 100%
					{
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ1, DoubleClick_100);
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ2, 0x00);
						Haptics_SendWaveform(LRA_ROM, ACTUATOR_LRA, TRIGGER_EXTERNAL_LEVEL);
						CapTouch_RepeatOnHoldWait();
						break;
					}
					default: __no_operation();
					}
				}
				else if(CapTouch_mode == MODE2)  	// Auto-resonance and ERM Alerts
				{
					switch(buttonPtr->referenceNumber)
					{
					case BUTTON1:	// LRA Auto-resonance ON (Closed-loop)
					{
						Haptics_SendWaveform(RTP_Alert, ACTUATOR_LRA, TRIGGER_INTERNAL);
						CapTouch_RepeatOnHoldWait();
						break;
					}
					case BUTTON2:	// LRA Auto-resonance OFF (Open-loop)
					{
						Haptics_SetActuator(ACTUATOR_LRA);					// Set actuator type
						Haptics_Start(PWM_Alert_NoBrake.inputMode);			// Enable the amplifier, begin PWM (if required)
						Haptics_HardwareMode(PWM_Alert_NoBrake.inputMode);	// Configure the Hardware (load switch, GPIOs)
						Haptics_SendActuatorSettings();						// Send actuator settings to DRV260x

						I2C_WriteSingleByte(DRV260x_OD_CLAMP, Voltage_2p3);							// Set overdrive clamp
						I2C_WriteSingleByte(DRV260x_CONTROL3, LRA_OpenLoop | NGThresh_4PERCENT);	// Set LRA open loop

						Haptics_OutputWaveform(PWM_Alert_NoBrake);			// Send waveform data to driver

						Haptics_Stop(PWM_Alert_NoBrake.inputMode);			// Stop PWM when finished

						CapTouch_RepeatOnHoldWait();
						break;
					}
					case BUTTON3:	// ERM Alert
					{
						Haptics_SetERMOpenLoopMode(0);
						Haptics_SendWaveform(RTP_Alert, ACTUATOR_ERM, TRIGGER_INTERNAL);
						CapTouch_RepeatOnHoldWait();
						break;
					}
					case BUTTON4:
					{
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ1, BuzzAlert750ms);
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ2, 0x00);
						//Haptics_SendWaveform(ERM_ROM, ACTUATOR_ERM, TRIGGER_INTERNAL);

						/* Output Waveform */
						Haptics_SetActuator(ACTUATOR_ERM);					// Set actuator type
						Haptics_SetTrigger(TRIGGER_INTERNAL);				// Set trigger variable to internal trigger
						Haptics_Start(ERM_ROM.inputMode);					// Enable the amplifier, begin PWM (if required)
						Haptics_HardwareMode(ERM_ROM.inputMode);			// Configure the Hardware (load switch, GPIOs)
						Haptics_SendActuatorSettings();						// Send actuator settings to DRV260x
						Haptics_SendTriggerType();							// Send trigger type to DRV2605
						I2C_WriteSingleByte(DRV260x_OD_CLAMP, Voltage_3p0);	// Set overdrive clamp
						Haptics_OutputWaveform(ERM_ROM);					// Send waveform data to driver
						break;
					}
					default: __no_operation();
					}
				}
				else if(CapTouch_mode == MODE1)   	// Simon Game
				{
					#ifdef SIMONGAME_H_
					Simon_Play(buttonPtr->sequenceNumber);
					#endif
				}
				else if(CapTouch_mode == MODE0)		// Audio2Haptics
				{
					CapTouch_ButtonLEDsOff();

					switch(buttonPtr->referenceNumber)
					{
					case BUTTON1:
					{
						if(a2h_setting == 1)	// If A2H is enabled, disable
						{
							P1OUT = 0;
							Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ1, StrongClick_100);
							Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ2, 0x00);
							Haptics_SendWaveform(ERM_ROM, ACTUATOR_ERM, TRIGGER_INTERNAL);
							a2h_setting = 0;
						}
						else					// Enable A2H (ERM)
						{
							P1OUT = BUTTON1;
							Haptics_SendWaveform(A2H_AC, ACTUATOR_ERM, TRIGGER_INTERNAL);
							a2h_setting = 1;
							Haptics_EffectActive(0);
						}
						break;
					}
					case BUTTON2:
					{
						if(a2h_setting == 2)	// If A2H is enabled, disable
						{
							P1OUT = 0;
							Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ1, StrongClick_100);
							Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ2, 0x00);
							Haptics_SendWaveform(LRA_ROM, ACTUATOR_LRA, TRIGGER_INTERNAL);
							a2h_setting = 0;
						}
						else					// Enable A2H (LRA)
						{
							Haptics_SendWaveform(A2H_AC, ACTUATOR_LRA, TRIGGER_INTERNAL);
							Haptics_EffectActive(0);
							P1OUT = BUTTON2;
							a2h_setting = 2;
						}
						break;
					}
					case BUTTON3:	// Play effect and then return to A2H
					{
						P1OUT = BUTTON3;
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ1, StrongClick_100);
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ2, 0x00);
						Haptics_SendWaveformDefault(Generic_ROM);

						Haptics_WaitWaveformComplete();		// Wait for waveform to complete

						/* Re-enable to Audio-to-Haptics */
						if(a2h_setting == 1)
						{
							P1OUT = BUTTON1;
							Haptics_SendWaveformDefault(A2H_AC);
							Haptics_EffectActive(0);
						}
						else if(a2h_setting == 2)
						{
							P1OUT = BUTTON2;
							Haptics_SendWaveformDefault(A2H_AC);
							Haptics_EffectActive(0);
						}

						break;
					}
					case BUTTON4:	// Play effect and then return to A2H
					{
						P1OUT = BUTTON4;
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ1, BuzzAlert750ms);
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ2, 0x00);
						Haptics_SendWaveformDefault(Generic_ROM);

						Haptics_WaitWaveformComplete();		// Wait for waveform to complete

						/* Re-enable to Audio-to-Haptics */
						if(a2h_setting == 1)
						{
							P1OUT = BUTTON1;
							Haptics_SendWaveformDefault(A2H_AC);
							Haptics_EffectActive(0);
						}
						else if(a2h_setting == 2)
						{
							P1OUT = BUTTON2;
							Haptics_SendWaveformDefault(A2H_AC);
							Haptics_EffectActive(0);
						}
					}
					default: __no_operation();
					}
					CapTouch_FreezeLEDs();
				}
			}
			else
			{
				#ifdef BINARYMODES_H_
					/* Include extra binary modes - life test, external PWM mode, etc. */
					BinaryModes();
				#endif
				#ifdef LIBRARYMODES_H_
					LibraryModes();
				#endif
			}
		}
    } // End ButtonPtr
	else	// else effect button is released
	{
		if(!CapTouch_isEffectBtnReleased) // Up button effects for MODE2
		{
			if(CapTouch_GetBinaryMode() == 0)	// Check if program is on top level
			{
				if(CapTouch_mode == MODE4)		// ERM Clicks Release Events
				{
					switch(lastButtonPtr->referenceNumber)
					{
					case BUTTON1:
					{
						break;
					}
					case BUTTON2:
					{
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ1, SharpClick_100);
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ2, 0x00);
						Haptics_SendWaveform(ERM_ROM, ACTUATOR_ERM, TRIGGER_INTERNAL);
						break;
					}
					case BUTTON3:
					{

						break;
					}
					case BUTTON4:
					{

						break;
					}
					default: __no_operation();
					}
				}
				else if(CapTouch_mode == MODE3)		// LRA Click Release Events
				{
					switch(lastButtonPtr->referenceNumber)
					{
					case BUTTON1:
					{
						break;
					}
					case BUTTON2:
					{
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ1, SharpTick2_80);
						Haptics_SetSequenceRegister(DRV260x_WAVEFORMSEQ2, 0x00);
						Haptics_SendWaveform(LRA_ROM, ACTUATOR_LRA, TRIGGER_INTERNAL);
						break;
					}
					case BUTTON3:
					{

						break;
					}
					case BUTTON4:
					{

						break;
					}
					default: __no_operation();
					}
				}
			}
			else
			{
				#ifdef BINARYMODES_H_
					BinaryModesRelease(); 	// Release functions for extra binary modes
				#endif
			}
		}

		CapTouch_RepeatReset();					// Reset status and counters when repeat mode is finished
		CapTouch_isEffectBtnReleased = 1;		// Button was released
	}

    if(Haptics_IsEffectActive())	// If effect is playing
    {
    	if(I2C_ReadSingleByte(DRV260x_GO) != GO)	// Check if GO bit is high
    	{
    		Haptics_Stop(0);			// Disable and reset haptics
    	}
    	else
    	{
			timerdelay(10000);
    	}
    }

    //sleep(500);		// Set CapTouch Polling Rate

  } 	// End While Loop
} 	// End Main

/******************************************************************************/
// Timer0_A0 Interrupt Service Routine:       
/******************************************************************************/
#pragma vector=TIMER0_A0_VECTOR
__interrupt void ISR_Timer0_A0(void)
{
  TA0CTL &= ~(MC_1);
  TA0CCTL0 &= ~(CCIE);

  CapTouch_RandomNumber++;

  __bic_SR_register_on_exit(LPM0_bits+GIE);
}


#pragma vector=TIMER1_A1_VECTOR
__interrupt void Timer_A1_ISR (void)
{
  TA1CCTL1 &= ~CCIFG;   //Clear Interrupt Flag
}
